;********************************************************************
;--------------------------------------------------------------------
;
;    Titolo        :  Indicatore temperatura per dissipatori
;                     PIC10F220/222  + sensore MCP9701
;    Data          :  11-03-2014
;    Modificato il :
;    Versione      :  V0.0
;    Ref. hardware :
;    Autore        :  afg
;
;--------------------------------------------------------------------
;********************************************************************
;
;  Impiego pin :
;  ----------------
;       10F220/2 @ 8 pin DIP
;
;                  |\/|
;              NC -|1    8|- GP3
;             Vdd -|2    7|- Vss
;             GP2 -|3    6|- NC
;             GP1 -|4    5|- GP0
;                  |______|
;
;       10F220/2 @ 6 pin SOT-23
;
;                  *|
;             GP0 -|1    6|- GP3
;             Vss -|2    5|- Vdd
;             GP1 -|3    4|- GP2
;                  |______|
;
;                           DIP  SOT
;    NC                      1:     nc
;    Vdd                     2:  5: ++
;    GP2/T0CKI/FOSC4         3:  4: Out    LED Verde
;    GP1/AN1/ICSPCLK         4:  3: Out    LED Rosso
;    GP0/AN0/ICSPDAT         5:  1: Analog In
;    NC                      6:     nc
;    Vss                     7:  2: --
;    GP3/MCLR/VPP            8:  6:
;
;
;********************************************************************
;####################################################################
;
;    Note :
;
;********************************************************************
;####################################################################
; Processor selection
 #ifdef  __10F220

       list        p=10F220 , r = DEC
       #include "p10f220.inc"
 #endif

 #ifdef  __10F222

         list      p=10F222 , r = DEC
        #include "p10f222.inc"
 #endif

;####################################################################
;
;====================================================================
;                          CONFIGURAZIONE                           =
;====================================================================
; Esempio di CONFIG
; -----------------
  __config  _CP_OFF & _MCLRE_ON & _IOFSCS_4MHZ & _MCPU_ON  & _WDT_OFF


;####################################################################

;********************************************************************
;*                   ASSEGNAZIONI LOCALI                            *
;********************************************************************
; Uscita LMCP9701 19.5 mV/C + 400mV
;             temp     Vout    ADCdec  ADChex
k1 equ .60  ; 40C -> 1.180V ->  60 -> 3Ch
k2 equ .70  ; 50C -> 1.375V ->  70 -> 46h
k3 equ .80  ; 60C -> 1.570V ->  80 -> 50h
k4 equ .90  ; 70C -> 1.765V ->  90 -> 5Ah
k5 equ .100 ; 80C -> 1.960V -> 100 -> 64h

;====================================================================
;            DEFINIZIONE DI IMPIEGO DEI PORT
;====================================================================
;sGPIO map
; |  3  |  2  |  1  |  0  |
; |-----|-----|-----|-----|
; | in  | LEDG| LEDR| AN0 |
;
#define    LEDR  sGPIO,GP1   ; LED Rosso tra pin e Vss
#define    LEDV  sGPIO,GP2   ; LED Verde
;#define    GPIO,GP0    ; AN0
;#define    GPIO,GP3    ; solo input

;####################################################################
;====================================================================
;=                        MEMORIA RAM                               =
;====================================================================
; RAM senza banchi

 #ifdef __10F222
;Zona di 24 bytes
;----------------
     CBLOCK 0x09             ; area RAM da 0x08 a 0x1F
        sGPIO                ; shadow I/O
        d1                   ; contatori per Delay
        d2
        d3
     ENDC
 #else                       ; 10F220 - meno RAM
;Zona di 16 bytes
;----------------
     CBLOCK 0x10             ; area RAM da 0x10 a 0x1F
        sGPIO                ; shadow I/O
        d1                   ; contatori per Delay
        d2
        d3
     ENDC
 #endif
;====================================================================
;=                        LOCAL MACROS                              =
;====================================================================

; local MACRO
EXEC  MACRO
        movf    sGPIO, W
        movwf   GPIO
      ENDM
; accende led Verde
Verde_on  MACRO
   bsf   LEDV
          ENDM
; accende led Rosso
Rosso_on  MACRO
   bsf   LEDR
          ENDM
; spegne led Verde
Verde_of  MACRO
   bcf   LEDV
          ENDM
; spegne led Rosso
Rosso_of  MACRO
   bcf   LEDR
          ENDM
; toggle rosso
Rosso_tg MACRO
   movlw  b'00010'   ; toggle rosso
   xorwf  sGPIO, f
         ENDM
; toggle verde
Verde_tg MACRO
   movlw  b'00100'   ; toggle verde
   xorwf  sGPIO, f
         ENDM
 PAGE

;####################################################################
;====================================================================
;=                         RESET ENTRY                              =
;====================================================================
; Sicurezza per il valore di calibrazione
RCCAL   ORG     0x1FF
        res     1

; Reset Vector
        ORG     0x00

; calibrazione oscillatore interno
        andlw   0xFE             ; clear bit0, no Fosc out
        movwf   OSCCAL


;####################################################################
;====================================================================
;=                        MAIN PROGRAM                              =
;====================================================================
MAIN:
; inizializzazioni I/O al reset
; OPTION default  '11111111'
        movlw    b'11010111'
        ;          1-------    GPWU disabilitato
        ;          -1------    GPPU disabilitato
        ;          --0-----    clock interno
        ;          ---1----    done
        ;          ----0---    prescaler al timer
        ;          -----111    1:256
        OPTION                 ; al registro OPTION

; ADCON0 default  '11001100'
        movlw    b'01000001'   ; ADC abilitato, GP0 = AN0
        ;          0-------    ANS1
        ;          -1------    ANS0
        ;          --00----    0
        ;          ----00--    CH 0
        ;          ------0-    GO
        ;          -------1    ADC enabled
        movwf   ADCON0

; GPIO
        clrf    GPIO            ; preset GPIO latch a 0
        movlw   b'1001'         ; GP1/2 = out
        tris    GPIO            ; al registro direzione

;----------------------------------------------------------
;--------------------- mainloop ---------------------------
;----------------------------------------------------------
   Verde_of           ; LED off
   Rosso_of
   EXEC

ml call   ReadAD      ; read ADC

;range evaluation
   movlw  k1          ; < k1?
   subwf  ADRES, w
   skpc
   goto   sel1
n1 movlw  k2          ; < k2?
   subwf  ADRES, w
   skpc
   goto   sel2
n2 movlw  k3          ; < k3?
   subwf  ADRES, w
   skpc
   goto   sel3
n3 movlw  k4          ; < k4?
   subwf  ADRES, w
   skpc
   goto   sel4
n4 goto   sel5        ; > k4?

; LED driving
sel1:
   Verde_on          ; T<40C verde fisso
   Rosso_of
   goto   selxec
sel2:                ; 40<T<50C lampeggio verde/giallo
   Verde_on          ; verde fisso +
   movlw  b'00010'   ; toggle rosso
   xorwf  sGPIO, f
   goto   selxec
sel3:                ; 50<T<60C giallo fisso
   Verde_on          ; verde + rosso
   Rosso_on
   goto   selxec
sel4:                ; 60<T<70C lampeggio giallo/rosso
   Rosso_on          ; rosso fisso +
   movlw  b'00100'   ; toggle verde
   xorwf  sGPIO, f
   goto   selxec
sel5:                ; T>70C rosso fisso
   Verde_of
   Rosso_on

selxec EXEC          ; drive LED

   call   Delay025s  ; 250ms delay
   call   Delay025s  ; 250ms delay

   goto   ml         ; close loop


;********************************************************************
;====================================================================
;=                          SUBROUTINES                             =
;====================================================================
;********************************************************************
; Read ADC
ReadAD  bsf         ADCON0, 1    ; start conversion
loopADC
        nop
        btfsc       ADCON0, 1    ; wait for DONE
        goto        loopADC
        retlw       0

; Delay = 0.25 seconds
; Clock frequency = 4 MHz
; 0.25 seconds = 250000 cycles
Delay025s              ;249993 cycles
        movlw   0x4E
        movwf   d1
        movlw   0xC4
        movwf   d2
Delay025s_0
        decfsz  d1, f
        goto    $+2
        decfsz  d2, f
        goto    Delay025s_0
        goto    $+1     ;3 cycles
        nop
        retlw   0       ;4 cycles (including call)

;********************************************************************
; ===================================================================
; =                           THE END                               =
;====================================================================
;********************************************************************
        END

 PAGE
;********************************************************************
; ===================================================================
; =                        DOCUMENTAZIONE                           =
;====================================================================
;********************************************************************
; Opzioni CONFIG
;
; Protezione del programma
; -------------------------
;_CP_ON                 protezione abilitata
;_CP_OFF                protezione disabilitata

; Pin MCLR
; ---------
;_MCLRE_ON              GP3/MCLR come MCLR;
;_MCLRE_OFF             GP3/MCLR come GP3

; Pull-up MCLR
; -------------
;_MCPU_ON               abilitato
;_MCPU_OFF              disabilitato

; Watchdog
; ---------
;_WDT_ON                abilitato
;_WDT_OFF               disabilitato
;_WDTE_ON               alias
;_WDTE_OFF              alias

; Internal Oscillator
; --------------------
;_IOFSCS_4MHZ           4 MHz
;_IOFSCS_8MHZ           8 MHz
;_IOSCFS_4MHZ           alias
;_IOSCFS_8MHZ           alias

;********************************************************************
;                        PRINCIPALI SFR                             *
;********************************************************************
; SFR senza banchi
;
; Esempio di maschera di inizializzazione e
; descrizione sintetica dei registri
;
;OPTION_REG - Wake pull-up & TIMER0
;-----------------------------------
; al POR          '11111111'
OPTIONVAL   EQU  B'11010111'
        ;          1-------     GPWU disabilitato
        ;          -1------     GPPU disabilitato
        ;          --0-----     clock interno
        ;          ---1----     done
        ;          ----0---     prescaler al timer
        ;          -----111     1:256
        ;
        ; NOT_GPWU  b7 : enable wake-up pin on change
        ;                1= disabilitato
        ;                0= abilitato
        ; NOT_GPPU  b6 : Enable Weak Pull-ups bit
        ;                1= disabilitato
        ;                0= abilitato
        ; TOCS      b5 : 1= clock TMR0 esterno da GP2
        ;                0= clock interno
        ; TOSE      b4 : 1= GP2 fronte discendente (b5=1)
        ;                0= GP2 fronte in salita   (b5=1)
        ; PSA       b3 : 1= predivisore al Watchdog
        ;                0= predivisore a TMR0
        ; PS2/PS0   b2/b0 valore del predivisore
        ;                  Watchdog   Timer0
        ;           000 =  1/1        1/2
        ;           001 =  1/2        1/4
        ;           010 =  1/4        1/8
        ;           011 =  1/8        1/16
        ;           100 =  1/16       1/32
        ;           101 =  1/32       1/64
        ;           110 =  1/64       1/128
        ;           111 =  1/128      1/256

;ADCON0 - AD converter
;----------------------
; ADCON0 default  '11001100'
ADCON0VAL   EQU  B'01001100'
        ;          0-------    ANS1
        ;          -1------    ANS0
        ;          --00----    0
        ;          ----11--    CH 0.6V
        ;          ------0-    no GO
        ;          -------0    ADC disabled
        ;
        ; ANS1    b7   : selziona AN1/GP1
        ;                1= AN1
        ;                0 = GP1
        ; ANS0    b6   : selziona AN0/GP0
        ;                1 = AN0
        ;                0 = GP0
        ; res     b5:4   00
        ; CHS1:0  b3:2   channel select
        ;                00 = AN0
        ;                01 = AN1
        ;                1x = Vref 0.6V
        ; GO/DONE   b1 : 1 = go / conv. in corso
        ;                0 = conversione conclcusa
        ; ADON      b0 : 1 = ADC on
        ;                0 = ADC off


; OSCCAL - Calibrazione oscillatore interno
; ------------------------------------------
; al POR           '11111110'
OSCCALDEF  EQU    B'00000000' ; valore intermedio

        ; CAL6:0    b7-1  : 0111111 = Maximum frequency
        ;                   0000000 = Center frequency
        ;                   1000000 = Minimum frequency
        ; FOSC4     b0    : 1= INTOSC/4 su GP2
        ;                   0= GP2/T0CKI/COUT


; Ordine di precedenza delle funzioni dei PORT
; Priorit | GP0  | GP1  | GP2  | GP3
;-----------------|------|------|-------
;   1      | AN0  | AN1  | FOSC4| I/MCLR
;   2      | TRIS | TRIS | T0CKI|  -
;   3      |  -   |  -   | TRIS |  -

; GPIO - Registro PORT
;----------------------
; al POR          '----xxxx'

; TRISGPIO - Direzione GPIO
; --------------------------
; al POR          '----1111'
DIRPORT    EQU   B'----1110'
         ;         |  |||||____ TRISGP0 in
         ;         |  ||||_____ TRISGP1 in
         ;         |  |||______ TRISGP2 in
         ;         |  ||_______ TRISGP3 solo in
         ;         |__|________ -